/* ---------------------------------------------------------------------------
created on: 06.04.2007
    by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * <p>Represents a command line as given by the user to start the
 * application.</p>
 *
 * <p>The commando itself is the first thing entered in the command line. It is
 * followed by options and arguments.</p>
 *
 * <p>The command line is invalid by default. The only way to get a valid
 * command line is to fill it with valid parameters at construction.</p>
 *
 * <dl>
 *  <dt>Invariants</dt>
 *  <dd>this.commando != <code>null</code></dd>
 *  <dd>this.parameters != <code>null</code></dd>
 *  <dd>this.helpText != <code>null</code></dd>
 *  <dd>this.line != <code>null</code></dd>
 * </dl>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: CmdLn.java,v 1.11 2007/05/01 17:08:22 GoerntKai Exp $
 */
class CmdLn implements CommandLine {

    /**
     * <p>Holds the commando.</p>
     */
    private String commando = null;

    /**
     * <p>Holds the help text.</p>
     */
    private String[] helpText = null;

    /**
     * <p>Holds the line as it was given by the user.</p>
     */
    private String line = null;

    /**
     * <p>Holds the messages.</p>
     */
    private List messages = new ArrayList();

    /**
     * <p>Holds the parameters.</p>
     */
    private Map parameters = null;

    /**
     * <p>Holds the validity.</p>
     */
    private boolean validity = false;

    /**
     * <p>Standard constructor.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theCommando != <code>null</code></dd>
     *  <dd>theParameters != <code>null</code></dd>
     *  <dd>theHelpText != <code>null</code></dd>
     *  <dd>theLine != <code>null</code></dd>
     * </dl>
     *
     * @param theCommando given
     * @param theParameters given by the user
     * @param theHelpText describing the command line
     * @param theLine as given by the user
     */
    public CmdLn(
        final String theCommando,
        final Map theParameters,
        final String[] theHelpText,
        final String theLine
    ) {

        super();

        // preconditions
        if (theCommando == null) {
            throw new IllegalArgumentException(
                "The commando is [null]!" //$NON-NLS-1$
            );
        }
        if (theParameters == null) {
            throw new IllegalArgumentException(
                "The parameters are [null]!" //$NON-NLS-1$
            );
        }
        if (theHelpText == null) {
            throw new IllegalArgumentException(
                "The help text is [null]!" //$NON-NLS-1$
            );
        }
        if (theLine == null) {
            throw new IllegalArgumentException(
                "The line is [null]!" //$NON-NLS-1$
            );
        }

        this.commando = theCommando;
        this.parameters = theParameters;
        this.helpText = theHelpText;
        this.line = theLine;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Checks the invariants. As there are so many it is encapsuled in this
     * method.</p>
     *
     * @return <code>true</code>
     */
    private boolean checkInvariants() {

        // bewares of NullPointerException
        assert this.commando != null : "this.commando is [null]"; //$NON-NLS-1$
        assert this.parameters != null :
            "this.parameters are [null]"; //$NON-NLS-1$
        assert this.helpText != null : "this.helpText is [null]"; //$NON-NLS-1$
        assert this.line != null : "this.line is [null]"; //$NON-NLS-1$

        return true;
    }

    /**
     * <p>Returns the commando.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code> (Invariant)</dd>
     * </dl>
     *
     * @return the commando
     * @see de.goerntkai.toolbox.cmdln.CommandLine#getCommando()
     */
    public String getCommando() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return this.commando;
    }

    /**
     * <p>Returns the text of the help document. In case of an invalid command
     * line the help text should begin with the command line followed by the
     * mistakes.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code> (Invariant)</dd>
     * </dl>
     *
     * @return the text of the help document
     * @see de.goerntkai.toolbox.cmdln.CommandLine#getHelpText()
     */
    public String[] getHelpText() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        List text = new ArrayList();
        if (!this.isValid()) {
            text.add("You asked for:"); //$NON-NLS-1$
            text.add("    " + this.line); //$NON-NLS-1$
            text.add(""); //$NON-NLS-1$
            text.add(this.commando + " says:"); //$NON-NLS-1$
            for (Iterator iterator = this.messages.iterator(); iterator
                .hasNext();) {

                String message = (String) iterator.next();
                text.add("    - " + message); //$NON-NLS-1$
            }
            text.add(""); //$NON-NLS-1$
        }
        String[] result = new String[text.size() + this.helpText.length];
        for (int idx = 0; idx < result.length; idx++) {
            if (idx < text.size()) {
                result[idx] = (String) text.get(idx);
            } else {
                result[idx] = this.helpText[idx - text.size()];
            }
        }

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return result;
    }

    /**
     * <p>Returns the command line parameter with the given name.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theName != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code> (Invariant)</dd>
     * </dl>
     *
     * @param theName of the command line parameter to get
     * @return the command line parameter
     * @see de.goerntkai.toolbox.cmdln.CommandLine#getParameter(
     *      java.lang.String)
     */
    public CommandLineParameter getParameter(final String theName) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theName == null) {
            throw new IllegalArgumentException(
                "The parameter name is [null]!" //$NON-NLS-1$
            );
        }

        CommandLineParameter result = (
            (CommandLineParameter) this.parameters.get(theName)
        );

        // postconditions
        assert result != null : "result is [null]"; //$NON-NLS-1$

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return result;
    }

    /**
     * <p>Returns <code>true</code> when the given command line was valid,
     * <code>false</code> otherwise.</p>
     *
     * @return wether the command line is valid; <code>this.validity</code>
     * @see de.goerntkai.toolbox.cmdln.CommandLine#isValid()
     */
    public boolean isValid() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return this.validity;
    }

    /**
     * <p>Sets the command line invalid.</p>
     *
     * @param theMessage why it is invalid
     */
    public void setInvalid(final String theMessage) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        this.validity = false;
        this.messages.add(theMessage);
    }

    /**
     * <p>Validates the command line. Adds invalidity messages to show to the
     * user.</p>
     */
    public void validate() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        boolean result = true;

        for (
            Iterator iterator = this.parameters.values().iterator();
            iterator.hasNext();
        ) {

            InternalParameter element = (InternalParameter) iterator.next();
            if (!element.isValid()) {
                result = false;
                this.messages.add(
                    "The argument [" + //$NON-NLS-1$
                    element.getName() +
                    "] is mandatory!" //$NON-NLS-1$
                );
            }
        }

        this.validity = result;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }
}



/* end of file ------------------------------------------------------------ */
