/* ---------------------------------------------------------------------------
created on: 01.04.2007
    by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.goerntkai.toolbox.fs.File;
import de.goerntkai.toolbox.fs.Folder;
import de.goerntkai.toolbox.fs.TextReader;
import de.goerntkai.toolbox.time.Clock;
import de.goerntkai.toolbox.time.ClockFactory;

/**
 * <p>Creates <code>CommandLine</code>s.</p>
 *
 * <p>Logs in level CONFIG.</p>
 *
 * <dl>
 *  <dt>Invariants</dt>
 *  <dd>log != <code>null</code></dd>
 * </dl>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version
 *      $Id: CommandLineFactory.java,v 1.12 2007/05/06 09:04:38 GoerntKai Exp $
 */
public abstract class CommandLineFactory {

    /**
     * <p>Holds the extension for the help document.</p>
     */
    private static final String HELPDOC_EXTENSION = ".help"; //$NON-NLS-1$

    /**
     * <p>Holds the log.</p>
     */
    private static Logger log = Logger.getLogger(
        CommandLineFactory.class.getName()
    );

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private CommandLineFactory() {

        // nop
    }

    /**
     * <p>Creates a <code>CommandLine</code> with the parameters given in the
     * command line. The commando must be given as a seperate argument.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theCommando != <code>null</code></dd>
     *  <dd>theParameters != <code>null</code></dd>
     *  <dd>theDirectory != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theCommando itself
     * @param theParameters of the command line
     * @param theDirectory where to find the help document in
     * @return a new <code>CommandLine</code> based on the command line
     * @throws FileNotFoundException when the help document was not found
     * @throws HelpTextException when the help document is invalid
     */
    private static CommandLine create(
        final String theCommando,
        final String[] theParameters,
        final Folder theDirectory
    ) throws FileNotFoundException, HelpTextException {

        // preconditions
        if (theCommando == null) {
            throw new IllegalArgumentException(
                "The commando is [null]!" //$NON-NLS-1$
            );
        }
        if (theParameters == null) {
            throw new IllegalArgumentException(
                "The parameters are [null]!" //$NON-NLS-1$
            );
        }
        if (theDirectory == null) {
            throw new IllegalArgumentException(
                "The folder is [null]!" //$NON-NLS-1$
            );
        }

        String method = "create"; //$NON-NLS-1$
        log.entering(CommandLineFactory.class.getName(), method);
        Clock clock = ClockFactory.createStopWatch();

        // loads the help text
        File help = new File(theDirectory, theCommando + HELPDOC_EXTENSION);
        log.config(
            "Loading help file [" + //$NON-NLS-1$
            help.getAbsolutePath() +
            "]." //$NON-NLS-1$
        );
        String[] helpText = TextReader.getTextLines(help);

        // parses the help text
        InternalParameter[] definition = HelpTextParser.parse(helpText);
        log.config(
            "Help text parsed into [" + //$NON-NLS-1$
            definition.length +
            "] parameters." //$NON-NLS-1$
        );

        // parses the command line
        String message = null;
        Map parameters = new HashMap();
        try {
            parameters = CommandLineParser.parse(definition, theParameters);

        } catch (CommandLineException exception) {
            message = exception.getMessage();
            log.log(
                Level.WARNING,
                "Caught CommandLineException.", //$NON-NLS-1$
                exception
            );
        }

        StringBuffer line = new StringBuffer();
        line.append(theCommando);
        for (int idx = 0; idx < theParameters.length; idx++) {
            line.append(' ');
            line.append(theParameters[idx]);
        }
        log.config(
            "User set [" + //$NON-NLS-1$
            theParameters.length +
            "] parameters in command line [" + //$NON-NLS-1$
            line.toString() +
            "]." //$NON-NLS-1$
        );
        log.config(
            "[" + //$NON-NLS-1$
            parameters.size() +
            "] parameters configured." //$NON-NLS-1$
        );

        CmdLn result = new CmdLn(
            theCommando,
            parameters,
            helpText,
            line.toString().trim()
        );
        if (message == null) {
            log.config(
                "Created a valid CommandLine in [" + //$NON-NLS-1$
                clock.getElapsedMillis() +
                "] ms." //$NON-NLS-1$
            );
        } else {
            result.setInvalid(message);
            log.config(
                "Created an invalid CommandLine in [" + //$NON-NLS-1$
                clock.getElapsedMillis() +
                "] ms." //$NON-NLS-1$
            );
        }

        // postconditions
        //assert result != null: "result is [null]"; //$NON-NLS-1$

        log.exiting(CommandLineFactory.class.getName(), method, result);
        return result;
    }

    /**
     * <p>Creates a <code>CommandLine</code> with the parameters given in the
     * command line. The commando must be given as a seperate argument.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theCommando != <code>null</code></dd>
     *  <dd>theParameters != <code>null</code></dd>
     *  <dd>theDirectory != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theCommando itself
     * @param theParameters of the command line
     * @param theDirectory where to find the help document in
     * @return a new <code>CommandLine</code> based on the command line
     * @throws FileNotFoundException when the help document was not found
     * @throws HelpTextException when the help document is invalid
     */
    public static CommandLine createCommandLine(
        final String theCommando,
        final String[] theParameters,
        final Folder theDirectory
    ) throws FileNotFoundException, HelpTextException {

        return create(theCommando, theParameters, theDirectory);
    }

    /**
     * <p>Creates a <code>CommandLine</code> with the parameters given in the
     * command line. The first parameter is expected to be the commando
     * itself.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theParameters != <code>null</code></dd>
     *  <dd>theParameters.length > 0</dd>
     *  <dd>theDirectory != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theParameters of the command line
     * @param theDirectory where to find the help document in
     * @return a new <code>CommandLine</code> based on the command line
     * @throws FileNotFoundException when the help document was not found
     * @throws HelpTextException when the help document is invalid
     */
    public static CommandLine createCommandLine(
        final String[] theParameters,
        final Folder theDirectory
    ) throws FileNotFoundException, HelpTextException {

        // preconditions
        if (theParameters == null) {
            throw new IllegalArgumentException(
                "The parameters are [null]!" //$NON-NLS-1$
            );
        }
        if (theParameters.length <= 0) {
            throw new IllegalArgumentException(
                "At least one parameter as the commando must" + //$NON-NLS-1$
                " be given!" //$NON-NLS-1$
            );
        }

        // reduces the parameters by the commando
        String[] reducedParms = new String[theParameters.length - 1];
        for (int idx = 0; idx < reducedParms.length; idx++) {
            reducedParms[idx] = theParameters[idx + 1];
        }

        return create(theParameters[0], reducedParms, theDirectory);
    }

    /**
     * <p>Resets the logger to its default value.</p>
     */
    public static void resetLog() {

        // invariants
        //assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

        log = Logger.getLogger(CommandLineFactory.class.getName());

        // invariants
        //assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Sets the log.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theLog != <code>null</code></dd>
     * </dl>
     *
     * @param theLog the log to set
     */
    public static void setLog(final Logger theLog) {

        // invariants
        //assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theLog == null) {
            throw new IllegalArgumentException(
                "The log is [null]!" //$NON-NLS-1$
            );
        }

        log = theLog;

        // invariants
        //assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$
    }
}



/* end of file ------------------------------------------------------------ */
