/* ---------------------------------------------------------------------------
created on: 06.04.2007
    by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

import java.util.HashMap;
import java.util.Map;

/**
 * <p>Parses the command line.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version
 *      $Id: CommandLineParser.java,v 1.4 2007/05/01 17:08:23 GoerntKai Exp $
 */
abstract class CommandLineParser {

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private CommandLineParser() {

        // nop
    }

    /**
     * <p>Finds an option with a given name in the parameter definition.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theName != <code>null</code></dd>
     *  <dd>theDefinition != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theName of the option to search for
     * @param theDefinition to search the option in
     * @return the option with the given name
     * @throws CommandLineException when the option is unknown
     */
    private static Option findOption(final String theName,
        final InternalParameter[] theDefinition) throws CommandLineException {

        // preconditions
        assert theName != null : "the name is [null]"; //$NON-NLS-1$
        assert theDefinition != null : "the definition is [null]"; //$NON-NLS-1$

        Option opt = null;
        for (int idx = theDefinition.length - 1; idx > 0; --idx) {
            InternalParameter parm = theDefinition[idx];
            if (parm instanceof Option && parm.getName().equals(theName)) {
                opt = (Option) parm;
            }
        }

        // postconditions
        if (opt == null) {
            throw new CommandLineException(
                "The option [" + //$NON-NLS-1$
                theName +
                "] is unknown!" //$NON-NLS-1$
            );
        }

        return opt;
    }

    /**
     * <p>Parses the command line parameters into their definition.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theDefinition != <code>null</code></dd>
     *  <dd>theParameters != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theDefinition of the command line parameters
     * @param theParameters given in the command line
     * @return all the possible parameters
     * @throws CommandLineException when the command line is not correct
     */
    public static Map parse(final InternalParameter[] theDefinition,
        final String[] theParameters) throws CommandLineException {

        // preconditions
        if (theDefinition == null) {
            throw new IllegalArgumentException(
                "The definition is [null]!" //$NON-NLS-1$
            );
        }
        if (theParameters == null) {
            throw new IllegalArgumentException(
                "The parameters are [null]!" //$NON-NLS-1$
            );
        }

        // creates the result
        Map result = new HashMap();

        int argCounter = 0;
        for (int idx = 0; idx < theParameters.length; idx++) {
            String parm = theParameters[idx];
            if (parm.charAt(0) == '-') {
                String name = parm.substring(1);
                Option opt = findOption(name, theDefinition);
                if (opt instanceof UnaryOption) {
                    opt.setValue(null);
                } else {
                    opt.setValue(theParameters[++idx]);
                }

            } else {
                theDefinition[argCounter++].setValue(parm);
            }
        }

        // moves parameters into the result map
        for (int idx = 0; idx < theDefinition.length; idx++) {
            InternalParameter parm = theDefinition[idx];
            result.put(parm.getName(), parm);
        }

        // postconditions
        //assert result != null: "result is [null]"; //$NON-NLS-1$

        return result;
    }
}



/* end of file ------------------------------------------------------------ */
