/* ---------------------------------------------------------------------------
created on: 04.04.2007
by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

/**
 * <p>Parses any given help text.</p>
 *
 * <p>The <code>InternalParameter</code>s are given in a certain order. At
 * first are the required and optional arguments as listed in the help text.
 * The options as listed in the help text follow.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: HelpTextParser.java,v 1.4 2007/04/29 17:54:03 GoerntKai Exp $
 */
abstract class HelpTextParser {

    /**
     * <p>Holds the parsing level for the description.</p>
     */
    private static final int LVL_DESCRIPTION = 2;

    /**
     * <p>Holds the parsing level for the optional arguments.</p>
     */
    private static final int LVL_OPTIONAL_ARGUMENTS = 4;

    /**
     * <p>Holds the parsing level for the options.</p>
     */
    private static final int LVL_OPTIONS = 5;

    /**
     * <p>Holds the parsing level for the prolog.</p>
     */
    private static final int LVL_PROLOG = 0;

    /**
     * <p>Holds the parsing level for the required arguments.</p>
     */
    private static final int LVL_REQUIRED_ARGUMENTS = 3;

    /**
     * <p>Holds the parsing level for the usage.</p>
     */
    private static final int LVL_USAGE = 1;

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private HelpTextParser() {

        // nop
    }

    /**
     * <p>Parses lines that belongs to the description block. Returns the
     * current parsing level.</p>
     *
     * <p>The parsing level is <code>3</code> when the line matches
     * <code>Required&nbsp;arguments:</code>. The parsing level is
     * <code>4</code> when the line matches
     * <code>Optional&nbsp;arguments:</code>. The parsing level is
     * <code>5</code> when the line matches <code>Options:</code>. Otherwise
     * it is <code>2</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theLine != <code>null</code></dd>
     *  <dd>theParts != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @param theParts of the parsed help text
     * @return the current level
     */
    private static int parseDescription(
        final String theLine,
        final ParsedHelpText theParts
    ) {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$
        assert theParts != null : "theParts are [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {
            if (theLine.matches("Required arguments:")) { //$NON-NLS-1$
                return LVL_REQUIRED_ARGUMENTS;
            }
            if (theLine.matches("Optional arguments:")) { //$NON-NLS-1$
                return LVL_OPTIONAL_ARGUMENTS;
            }
            if (theLine.matches("Options:")) { //$NON-NLS-1$
                return LVL_OPTIONS;
            }

            theParts.appendDescription(theLine);
        }

        return LVL_DESCRIPTION;
    }

    /**
     * <p>Parses lines that belongs to the block of optional arguments. Returns
     * the current parsing level.</p>
     *
     * <p>The parsing level is <code>5</code> when the line matches
     * <code>Options:</code>. Otherwise it is <code>4</code>.</p>
     *
     * <p>To identify the default value the line must match
     * <code>.*default: \\(.+\\)$</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions:</dt>
     *  <dd>theLine != <code>null</code></dd>
     *  <dd>theParts != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @param theParts of the parsed help text
     * @return the current level
     * @throws HelpTextException when the line is invalid
     */
    private static int parseOptionalArguments(
        final String theLine,
        final ParsedHelpText theParts
    ) throws HelpTextException {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$
        assert theParts != null : "theParts are [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {
            if (theLine.matches("Options:")) { //$NON-NLS-1$
                return LVL_OPTIONS;
            }

            if (!theLine.startsWith("{")) { //$NON-NLS-1$
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no starting bracket!" //$NON-NLS-1$
                );
            }

            int end = theLine.indexOf('}');
            if (end <= 2) {
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no arguments name!" //$NON-NLS-1$
                );
            }

            if (!theLine.matches(".*default: \\(.+\\)$")) { //$NON-NLS-1$
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no default value!" //$NON-NLS-1$
                );
            }
            int start = theLine.lastIndexOf('(');

            String name = theLine.substring(1, end);
            String value = theLine.substring(start + 1, theLine.length() - 1);
            theParts.addOptionalArgument(new Argument(name, value));
        }

        return LVL_OPTIONAL_ARGUMENTS;
    }

    /**
     * <p>Parses lines that belongs to the block of options. Returns the
     * current parsing level.</p>
     *
     * <p>The parsing level is <code>5</code>.</p>
     *
     * <p>To identify the given parameter, the line must match
     * <code>.*<\\S*>.*</code>.</p>
     *
     * <p>To identify the default value the line must match
     * <code>.*default: \\(.+\\)$</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions:</dt>
     *  <dd>theLine != <code>null</code></dd>
     *  <dd>theParts != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @param theParts of the parsed help text
     * @return the current level
     * @throws HelpTextException when the line is invalid
     */
    private static int parseOptions(
        final String theLine,
        final ParsedHelpText theParts
    ) throws HelpTextException {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$
        assert theParts != null : "theParts are [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {

            // checks starting minus
            if (theLine.charAt(0) != '-') {
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no starting minus!" //$NON-NLS-1$
                );
            }

            // checks starting bracket
            if (theLine.charAt(1) != '{') {
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no starting bracket!" //$NON-NLS-1$
                );
            }

            // gets the end of the options name
            int end = theLine.indexOf('}');
            if (end <= 2) {
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no options name!" //$NON-NLS-1$
                );
            }
            String name = theLine.substring(2, end);

            // gets the parameters name
            String paramName = null;
            String value = null;
            if (theLine.matches(".*<\\S*>.*")) { //$NON-NLS-1$
                int paramNameStart = theLine.indexOf("<"); //$NON-NLS-1$
                int paramNameEnd = theLine.lastIndexOf(">"); //$NON-NLS-1$
                paramName = theLine.substring(paramNameStart, paramNameEnd);

                // gets the start of the default value
                if (!theLine.matches(".*default: \\(.+\\)$")) { //$NON-NLS-1$
                    throw new HelpTextException("The line [" + //$NON-NLS-1$
                        theLine +
                        "] has no default value!" //$NON-NLS-1$
                    );
                }
                int start = theLine.lastIndexOf('(');
                value = theLine.substring(start + 1, theLine.length() - 1);
            }

            if (paramName == null) {
                theParts.addOption(new UnaryOption(name));

            } else {
                theParts.addOption(new BinaryOption(name, value));
            }
        }

        return LVL_OPTIONS;
    }

    /**
     * <p>Parses lines that are before the usage block. Returns the current
     * parsing level.</p>
     * <p>The parsing level is <code>1</code> when the line matches
     * <code>Usage:</code>. Otherwise it is <code>0</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions:</dt>
     *  <dd>theLine != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @return the current level
     * @throws HelpTextException when other text than <code>Usage:</code> or
     *      white spaces are found
     */
    private static int parseProlog(
        final String theLine
    ) throws HelpTextException {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {
            if (theLine.matches("Usage:")) { //$NON-NLS-1$
                return LVL_USAGE;
            }

            throw new HelpTextException(
                "The text must begin with usage block!" //$NON-NLS-1$
            );
        }

        return LVL_PROLOG;
    }

    /**
     * <p>Parses lines that belongs to the block of required arguments. Returns
     * the current parsing level.</p>
     * <p>The parsing level is <code>4</code> when the line matches
     * <code>Optional&nbsp;arguments:</code>. The parsing level is
     * <code>5</code> when the line matches <code>Options:</code>. Otherwise
     * it is <code>3</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions:</dt>
     *  <dd>theLine != <code>null</code></dd>
     *  <dd>theParts != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @param theParts of the parsed help text
     * @return the current level
     * @throws HelpTextException when the line is invalid
     */
    private static int parseRequiredArguments(
        final String theLine,
        final ParsedHelpText theParts
    ) throws HelpTextException {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$
        assert theParts != null : "theParts are [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {
            if (theLine.matches("Optional arguments:")) { //$NON-NLS-1$
                return LVL_OPTIONAL_ARGUMENTS;
            }
            if (theLine.matches("Options:")) { //$NON-NLS-1$
                return LVL_OPTIONS;
            }

            if (!theLine.startsWith("{")) { //$NON-NLS-1$
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no starting bracket!" //$NON-NLS-1$
                );
            }

            int end = theLine.indexOf('}');
            if (end <= 2) {
                throw new HelpTextException("The line [" + //$NON-NLS-1$
                    theLine +
                    "] has no arguments name!" //$NON-NLS-1$
                );
            }

            String name = theLine.substring(1, end);
            theParts.addRequiredArgument(new Argument(name, null));
        }

        return LVL_REQUIRED_ARGUMENTS;
    }

    /**
     * <p>Parses lines that belongs to the usage block. Returns the current
     * parsing level.</p>
     * <p>The parsing level is <code>2</code> when the line matches
     * <code>Description:</code>. Otherwise it is <code>1</code>.</p>
     *
     * <dl>
     *  <dt>Preconditions:</dt>
     *  <dd>theLine != <code>null</code></dd>
     *  <dd>theParts != <code>null</code></dd>
     * </dl>
     *
     * @param theLine the trimmed line
     * @param theParts of the parsed help text
     * @return the current level
     */
    private static int parseUsage(
        final String theLine,
        final ParsedHelpText theParts
    ) {

        // preconditions
        assert theLine != null : "theLine is [null]"; //$NON-NLS-1$
        assert theParts != null : "theParts are [null]"; //$NON-NLS-1$

        if (theLine.length() > 0) {
            if (theLine.matches("Description:")) { //$NON-NLS-1$
                return LVL_DESCRIPTION;
            }

            String[] parts = theLine.split(" "); //$NON-NLS-1$
            if (parts.length > 0) {
                theParts.setCommando(parts[0]);
            }
        }

        return LVL_USAGE;
    }

    /**
     * <p>Parses the given help text.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theText != <code>null</code></dd>
     *
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @param theText to parse
     * @return the parameter definition
     * @throws HelpTextException when description part is missing
     */
    public static InternalParameter[] parse(
        final String[] theText
    ) throws HelpTextException {

        // preconditions
        if (theText == null) {
            throw new IllegalArgumentException(
                "The help text is [null]!" //$NON-NLS-1$
            );
        }
        if (theText.length <= 0) {
            throw new HelpTextException(
                "The help text is empty! Usage block and" + //$NON-NLS-1$
                " description block are mandatory!" //$NON-NLS-1$
            );
        }

        // creates the result
        InternalParameter[] definition = null;

        // parse
        ParsedHelpText help = new ParsedHelpText();
        int parsingLevel = 0;
        for (int idx = 0; idx < theText.length; idx++) {
            String line = theText[idx];

            switch(parsingLevel) {
                case LVL_PROLOG:
                    parsingLevel = parseProlog(line.trim());
                    break;

                case LVL_USAGE:
                    parsingLevel = parseUsage(line.trim(), help);
                    break;

                case LVL_DESCRIPTION:
                    parsingLevel = parseDescription(line.trim(), help);
                    break;

                case LVL_REQUIRED_ARGUMENTS:
                    parsingLevel = parseRequiredArguments(line.trim(), help);
                    break;

                case LVL_OPTIONAL_ARGUMENTS:
                    parsingLevel = parseOptionalArguments(line.trim(), help);
                    break;

                case LVL_OPTIONS:
                    parsingLevel = parseOptions(line.trim(), help);
                    break;

                default:
                    break;
            }

            definition = help.getParameters();
        }

        // exception if no description given
        if (parsingLevel == LVL_USAGE) {
            throw new HelpTextException(
                "The text must have a description block after" + //$NON-NLS-1$
                " the usage block!" //$NON-NLS-1$
            );
        }

        // postconditions
        assert definition != null : "result is [null]"; //$NON-NLS-1$

        return definition;
    }
}



/* end of file ------------------------------------------------------------ */
