/* ---------------------------------------------------------------------------
created on: 06.04.2007
    by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>Holds the parsed parts of the help text. It exists only for the parsing
 * process.</p>
 *
 * <dl>
 *  <dt>Invariants</dt>
 *  <dd>this.description != <code>null</code></dd>
 *  <dd>this.requiredArguments != <code>null</code></dd>
 *  <dd>this.optionalArguments != <code>null</code></dd>
 *  <dd>this.options != <code>null</code></dd>
 * </dl>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: ParsedHelpText.java,v 1.2 2007/05/01 17:08:23 GoerntKai Exp $
 */
final class ParsedHelpText {

    /**
     * <p>Holds the commando.</p>
     */
    private String commando = null;

    /**
     * <p>Holds the description.</p>
     */
    private StringBuffer description = new StringBuffer();

    /**
     * <p>Holds the optional arguments.</p>
     */
    private List optionalArguments = new ArrayList();

    /**
     * <p>Holds the options.</p>
     */
    private List options = new ArrayList();

    /**
     * <p>Holds the required arguments.</p>
     */
    private List requiredArguments = new ArrayList();

    /**
     * <p>Checks the invariants. As there are so many it is encapsuled in this
     * method.</p>
     *
     * @return <code>true</code>
     */
    private boolean checkInvariants() {

        // bewares of NullPointerException
        assert this.description != null :
            "this.description is [null]"; //$NON-NLS-1$
        assert this.requiredArguments != null :
            "this.requiredArguments are [null]"; //$NON-NLS-1$
        assert this.optionalArguments != null :
            "this.optionalArguments are [null]"; //$NON-NLS-1$
        assert this.options != null : "this.options are [null]"; //$NON-NLS-1$

        return true;
    }

    /**
     * <p>Adds an option.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theOption != <code>null</code></dd>
     * </dl>
     *
     * @param theOption to set
     */
    public void addOption(final Option theOption) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theOption == null) {
            throw new IllegalArgumentException(
                "The option is [null]!" //$NON-NLS-1$
            );
        }

        this.options.add(theOption);

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Adds an optional argument.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theArgument != <code>null</code></dd>
     * </dl>
     *
     * @param theArgument to set
     */
    public void addOptionalArgument(final Argument theArgument) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theArgument == null) {
            throw new IllegalArgumentException(
                "The optional argument is [null]!" //$NON-NLS-1$
            );
        }

        this.optionalArguments.add(theArgument);

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Adds a required argument.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theArgument != <code>null</code></dd>
     * </dl>
     *
     * @param theArgument to set
     */
    public void addRequiredArgument(final Argument theArgument) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theArgument == null) {
            throw new IllegalArgumentException(
                "The required argument is [null]!" //$NON-NLS-1$
            );
        }

        this.requiredArguments.add(theArgument);

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Appends text to the description. In case that the description has
     * already text a seperating space character is appended first.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theDescription != <code>null</code></dd>
     * </dl>
     *
     * @param theDescription the description to set
     */
    public void appendDescription(final String theDescription) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theDescription == null) {
            throw new IllegalArgumentException(
                "The description is [null]!" //$NON-NLS-1$
            );
        }

        if (this.description.length() > 0) {
            this.description.append(" "); //$NON-NLS-1$
        }
        this.description.append(theDescription);

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Returns the commando.</p>
     *
     * @return <code>this.commando</code>
     */
    public String getCommando() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return this.commando;
    }

    /**
     * <p>Returns the description.</p>
     *
     * @return <code>this.description</code>
     */
    /*public String getDescription() {

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     return this.description.toString();
     }*/

    /**
     * <p>Returns the optional arguments.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @return <code>this.optionalArguments</code>
     */
    /*public Argument[] getOptionalArguments() {

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     Argument[] args = new Argument[this.optionalArguments.size()];

     // postconditions
     //assert args != null: "result is [null]"; //$NON-NLS-1$

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     return args;
     }*/

    /**
     * <p>Returns the options.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @return <code>this.options</code>
     */
    /*public Option[] getOptions() {

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     Option[] opts = new Option[this.options.size()];

     // postconditions
     //assert opts != null: "result is [null]"; //$NON-NLS-1$

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     return opts;
     }*/

    /**
     * <p>Returns the required arguments, optional arguments and options which
     * are defined in the help text.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @return the arguments and options
     */
    public InternalParameter[] getParameters() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // creates some sums
        int numberOfReqs = this.requiredArguments.size();
        int numberOfArgs = numberOfReqs + this.optionalArguments.size();
        int numberOfParams = numberOfArgs + this.options.size();

        // creates the result
        InternalParameter[] params = new InternalParameter[numberOfParams];

        // fills the array
        for (int idx = 0; idx < params.length; idx++) {

            if (idx < numberOfReqs) {
                params[idx] = (Argument) this.requiredArguments.get(idx);

            } else if (idx < numberOfArgs) {
                params[idx] = (Argument) this.optionalArguments.get(
                    idx - numberOfReqs
                );

            } else {
                params[idx] = (Option) this.options.get(idx - numberOfArgs);
            }
        }

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return params;
    }

    /**
     * <p>Returns the required arguments.</p>
     *
     * <dl>
     *  <dt>Postconditions</dt>
     *  <dd>result != <code>null</code></dd>
     * </dl>
     *
     * @return <code>this.requiredArguments</code>
     */
    /*public Argument[] getRequiredArguments() {

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     Argument[] args = new Argument[this.requiredArguments.size()];

     // postconditions
     //assert args != null: "result is [null]"; //$NON-NLS-1$

     // invariants
     assert this.checkInvariants(): "invalid invariants"; //$NON-NLS-1$

     return args;
     }*/

    /**
     * <p>Sets the commando.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theCommando != <code>null</code></dd>
     * </dl>
     *
     * @param theCommando the commando to set
     */
    public void setCommando(final String theCommando) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if (theCommando == null) {
            throw new IllegalArgumentException(
                "The commando is [null]!" //$NON-NLS-1$
            );
        }

        this.commando = theCommando;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }
}



/* end of file ------------------------------------------------------------ */
