/* ---------------------------------------------------------------------------
created on: 01.04.2007
    by: GoerntKai

Copyright: (c) 2007 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.cmdln;

import de.goerntkai.toolbox.misc.EqualsUtil;
import de.goerntkai.toolbox.misc.HashCodeUtil;

/**
 * <p>Represents an unary command line option.</p>
 *
 * <p>An unary option can never be invalid.</p>
 *
 * <dl>
 *  <dt>Invariants</dt>
 *  <dd>this.name != <code>null</code></dd>
 * </dl>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: UnaryOption.java,v 1.6 2007/05/01 17:08:23 GoerntKai Exp $
 */
class UnaryOption implements Option {

    /**
     * <p>Holds the name.</p>
     */
    private String name = null;

    /**
     * <p>Holds the settlement. Is <code>false</code> as long as this argument
     * is not set by the user. When once <code>setValue</code> is called it
     * becomes <code>true</code>.</p>
     */
    private boolean settlement = false;

    /**
     * <p>Standard constructor.</p>
     *
     * <dl>
     * <dt>Preconditions</dt>
     *  <dd>theName != <code>null</code></dd>
     * </dl>
     *
     * @param theName of this unary option
     */
    public UnaryOption(final String theName) {

        super();

        // preconditions
        if (theName == null) {
            throw new IllegalArgumentException(
                "The name is [null]!" //$NON-NLS-1$
            );
        }

        this.name = theName;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$
    }

    /**
     * <p>Checks the invariants. As there are so many it is encapsuled in this
     * method.</p>
     *
     * @return <code>true</code>
     */
    private boolean checkInvariants() {

        // bewares of NullPointerException
        assert this.name != null : "this.name is [null]"; //$NON-NLS-1$

        return true;
    }

    /**
     * <p>This method is mainly contained for testing.</p>
     * <dl>
     *  <dt>no Condition</dt>
     * </dl>
     *
     * <p>Implemented on recommendation of <a
     * href="http://www.javapractices.com/Topic17.cjp">
     * http://www.javapractices.com/Topic17.cjp</a>.</p>
     *
     * @param otherObject the object to compare with
     * @return <code>true</code> if this object is the same as
     *      <code>other</code> and <code>false</code> otherwise
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(final Object otherObject) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // superclass object checks identity
        if (super.equals(otherObject)) {
            return true;
        }

        // returns false when other object is null; see rule no. 5
        // checks symmetry
        if (!(otherObject instanceof UnaryOption)) {
            return false;
        }

        // casts other to this class
        UnaryOption other = (UnaryOption) otherObject;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // tests every member
        return EqualsUtil.areEqual(this.name, other.name);
    }

    /**
     * <p>Returns the name of this unary option.</p>
     *
     * @return the name of this unary option
     * @see de.goerntkai.toolbox.cmdln.InternalParameter#getName()
     */
    public String getName() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return this.name;
    }

    /**
     * <p>Returns the value of this unary option. The value can only be
     * "true" or "false", a String representation of a boolean value.</p>
     *
     * @return the value of this unary option
     * @see de.goerntkai.toolbox.cmdln.CommandLineParameter#getValue()
     */
    public String getValue() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return Boolean.toString(this.settlement);
    }

    /**
     * <p>Implemented as <code>equals()</code> is implemented.</p>
     * <dl>
     *  <dt>no Condition</dt>
     * </dl>
     *
     * <p>Implemented on recommendation of <a
     * href="http://www.javapractices.com/Topic28.cjp">
     * http://www.javapractices.com/Topic28.cjp</a>.</p>
     *
     * @return the hashCode
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        int result = HashCodeUtil.SEED;

        //collect the contributions of various fields
        result = HashCodeUtil.hash(result, this.name);

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return result;
    }

    /**
     * <p>Returns <code>true</code> when this unary option was set in the
     * command line, <code>false</code> otherwise.</p>
     *
     * @return wether this unary option was set in the command line
     * @see de.goerntkai.toolbox.cmdln.CommandLineParameter#isSet()
     */
    public boolean isSet() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return this.settlement;
    }

    /**
     * <p>Returns <code>true</code>.</p>
     *
     * @return wether this unary option is validly used
     * @see de.goerntkai.toolbox.cmdln.InternalParameter#isValid()
     */
    public boolean isValid() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return true;
    }

    /**
     * <p>Sets this unary option to be set in the command line. The value is
     * totally ignored.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theValue != "false"</dd>
     * </dl>
     *
     * @param theValue to indicate this unary option was set in the command
     *      line
     * @see de.goerntkai.toolbox.cmdln.InternalParameter#setValue(
     *      java.lang.String)
     */
    public void setValue(final String theValue) {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        // preconditions
        if ("false".equals(theValue)) { //$NON-NLS-1$
            throw new IllegalArgumentException(
                "The value may be anything but [\"false\"]!" //$NON-NLS-1$
            );
        }

        this.settlement = true;

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

    }

    /**
     * <p>This method is mainly contained for debugging. It returns
     * <code>super.toString()</code> concatenated with the fields and its
     * values.</p>
     *
     * @return the String representation of the object
     * @see java.lang.Object#toString()
     */
    public String toString() {

        // invariants
        assert this.checkInvariants() : "invalid invariants"; //$NON-NLS-1$

        return super.toString() + "[" + //$NON-NLS-1$
            "name=" + this.name + //$NON-NLS-1$
            "]"; //$NON-NLS-1$
    }
}



/* end of file ------------------------------------------------------------ */
