/* ---------------------------------------------------------------------------
created on: 09.10.2004
    by: GoerntKai

Copyright: (c) 2004 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */
package de.goerntkai.toolbox.fs;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;

/**
 * <p>Represents a file object in the file system. The base
 * <code>java.io.File</code> is extended with <code>copyTo</code> which copies
 * the file to the given destination.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: File.java,v 1.3 2007/05/06 16:25:51 GoerntKai Exp $
 * @see java.io.File
 */
public class File extends java.io.File {

    /**
     * Holds an empty string.
     */
    private static final String EMPTY_STRING = ""; //$NON-NLS-1$

    /**
     * <p>Holds the serial version ID.</p>
     */
    private static final long serialVersionUID = 0L;

    /**
     * Seperates the bare name from the suffix. Will be the same for all OS.
     */
    public static final String SUFFIX_SEPERATOR = "."; //$NON-NLS-1$

    /**
     * <p>Creates a new <code>File</code> from an object of its superclass.</p>
     *
     * @param theFile to create
     */
    public File(final java.io.File theFile) {

        super(theFile.getAbsolutePath());
    }

    /**
     * <p>Inherited from <code>java.io.File</code>.</p>
     *
     * @param theParent to create the child in
     * @param theChild to create
     * @see java.io.File
     */
    public File(final java.io.File theParent, final String theChild) {

        super(theParent, theChild);
    }

    /**
     * <p>Inherited from <code>java.io.File</code>.</p>
     *
     * @param thePathname of the file
     * @see java.io.File
     */
    public File(final String thePathname) {

        super(thePathname);
    }

    /**
     * <p>Inherited from <code>java.io.File</code>.</p>
     *
     * @param theParent to create the child in
     * @param theChild to create
     * @see java.io.File
     */
    public File(final String theParent, final String theChild) {

        super(theParent, theChild);
    }

    /**
     * <p>Inherited from <code>java.io.File</code>.</p>
     *
     * @param theURI of the file
     * @see java.io.File
     */
    public File(final URI theURI) {

        super(theURI);
    }

    /**
     * <p>Copies this file object to the given destination file. Copies only
     * files, no directories. Uses <code>FileInputStream</code> for the source
     * and <code>FileOutputStream</code> for the destination. The copy will
     * take byte for byte. Returns the number of bytes that were copied.
     * Returns <code>-1</code> if the source was no file. Any existing file
     * will be overwritten.</p>
     * <p>In case that parent directories of the destination files are missing
     * this method will create them.</p>
     *
     * @param destination the destination file
     * @return the number of bytes that were copied
     * @throws IOException on problems with the streams
     */
    public final long copyTo(final File destination) throws IOException {

        // creates the parent directories of the destination file
        destination.getParentFile().mkdirs();

        // initializes the byte counter
        long byteCounter = 0;

        // copy only if the file is really a file not a directory
        if (this.isFile()) {

            // open source and destination streams
            FileInputStream in = new FileInputStream(this);
            FileOutputStream out = new FileOutputStream(destination);

            // the transfer byte
            int character = 0;

            // copy byte by byte
            while ((character = in.read()) != -1) {
                out.write(character);
                byteCounter++;
            }

            // close source and destination streams
            in.close();
            out.close();

        } else {
            // set the byte counter if the source is no file
            byteCounter--;
        }

        return byteCounter;
    }

    /**
     * <p>Copies this file object to the given destination file. Just calls
     * <code>copyTo(File)</code> with the <code>File</code> created from
     * <code>destinationFile</code>.</p>
     *
     * @param destinationFile where to copy the file to
     * @return the number of bytes that were copied
     * @throws IOException on problems with the streams
     * @see File#copyTo(File)
     */
    public final long copyTo(final String destinationFile) throws IOException {

        return this.copyTo(new File(destinationFile));
    }

    /**
     * <p>Returns the bare name of the file name. The last dot is the
     * splitting element. The bare name is the part left to the last dot. For
     * example, the file name is <em>file-0.1.txt</em> the bare name will be
     * <em>file-0.1</em>.</p>
     *
     * @return the bare name
     */
    public final String getBareName() {

        // initialize the bare name
        String bareName = null;

        // get the index of the last dot
        String name = this.getName();
        int index = name.lastIndexOf(SUFFIX_SEPERATOR);

        if (index < 0) {
            // the dot was not found
            bareName = name;
        } else if (index == 0) {
            // the dot is the first character
            bareName = EMPTY_STRING;
        } else {
            // the dot is in the string
            bareName = name.substring(0, index);
        }

        return bareName;
    }

    /**
     * <p>Returns the suffix of the file name. The last dot is the splitting
     * element. The suffix is the part right to the last dot. For example, the
     * file name is <em>file-0.1.txt</em> the suffix will be <em>txt</em>.</p>
     *
     * @return the file name suffix
     */
    public final String getSuffix() {

        // initialize the suffix
        String suffix = null;

        // get the index of the last dot
        String name = this.getName();
        int index = name.lastIndexOf(SUFFIX_SEPERATOR);

        if (index < 0) {
            // the dot was not found
            suffix = EMPTY_STRING;
        } else {
            // the dot is in the string
            suffix = name.substring(index + 1);
        }

        return suffix;
    }
}



/* end of file ------------------------------------------------------------ */
