/* ---------------------------------------------------------------------------
created on: 10.07.2004
    by: GoerntKai

Copyright: (c) 2004 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */
package de.goerntkai.toolbox.fs;

import java.io.IOException;
import java.net.URI;

/**
 * <p>Represents a folder object in the file system. The base
 * <code>de.goerntkai.devbase.tools.File</code> is extended with
 * <code>hasSubdirectories</code>, <code>deleteContents</code> and
 * <code>deleteWithContents</code>. The <code>copyTo</code> which copies the
 * folder to the given destination is overwritten.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: Folder.java,v 1.4 2007/05/06 16:25:51 GoerntKai Exp $
 * @see de.goerntkai.toolbox.fs.File
 */
public class Folder extends File {

    /**
     * <p>Holds the serial version ID.</p>
     */
    private static final long serialVersionUID = 0L;

    /**
     * <p>Inherited from <code>de.goerntkai.devbase.tools.File</code>.</p>
     *
     * @param theParent to create the child in
     * @param theChild to create
     * @see de.goerntkai.toolbox.fs.File
     */
    public Folder(final File theParent, final String theChild) {

        super(theParent, theChild);
    }

    /**
     * <p>Creates a new <code>Folder</code> from an object of its
     * superclass.</p>
     *
     * @param theFile to create
     * @throws NoFolderException when <code>myself</code> is no directory
     */
    public Folder(final java.io.File theFile) throws NoFolderException {

        super(theFile.getAbsolutePath());

        if (super.exists() && !super.isDirectory()) {
            throw new NoFolderException(
                "Path [" + //$NON-NLS-1$
                theFile.getAbsolutePath() +
                "] is no directory!" //$NON-NLS-1$
            );
        }
    }

    /**
     * <p>Inherited from <code>de.goerntkai.devbase.tools.File</code>.</p>
     *
     * @param thePathname of the file
     * @throws NoFolderException when <code>pathname</code> is no directory
     * @see de.goerntkai.toolbox.fs.File
     */
    public Folder(final String thePathname) throws NoFolderException {

        super(thePathname);

        if (super.exists() && !super.isDirectory()) {
            throw new NoFolderException(
                "Path [" + //$NON-NLS-1$
                thePathname +
                "] is no directory!" //$NON-NLS-1$
            );
        }
    }

    /**
     * <p>Inherited from <code>de.goerntkai.devbase.tools.File</code>.</p>
     *
     * @param theParent to create the child in
     * @param theChild to create
     * @see de.goerntkai.toolbox.fs.File
     */
    public Folder(final String theParent, final String theChild) {

        super(theParent, theChild);
    }

    /**
     * <p>Inherited from <code>de.goerntkai.devbase.tools.File</code>.</p>
     *
     * @param theURI of the file
     * @see de.goerntkai.toolbox.fs.File
     */
    public Folder(final URI theURI) {

        super(theURI);
    }

    /**
     * <p>Copies this folder object to the given destination file. Copies
     * files as well as directories recursively. Returns the number of bytes
     * that were copied. Returns <code>-1</code> if the source was no folder or
     * does not exist.</p>
     *
     * @param destination where to copy the folder to
     * @return the number of bytes that were copied
     * @throws IOException on problems with the streams
     * @see de.goerntkai.toolbox.fs.File#copyTo(java.lang.String)
     * @see de.goerntkai.toolbox.fs.Folder#copyTo(Folder)
     */
    public final long copyTo(final Folder destination) throws IOException {

        // return -1 if the source is no directory or does not exist at all
        if (!this.exists() || !this.isDirectory()) {
            return -1;
        }

        // initializes the byte counter
        long byteCounter = 0;

        // create the destination folder if it not yet exists
        if (!destination.exists()) {
            destination.mkdirs();
        }

        // initializes the array of the contending files and folders
        java.io.File[] files = this.listFiles();

        // iterate over the contending files and folders
        if (files != null) {
            for (int idx = 0; idx < files.length; idx++) {

                // compute the path of the destination file
                File destFile = new File(
                    destination.getAbsolutePath() +
                    java.io.File.separator +
                    files[idx].getName()
                );

                // copy file
                if (files[idx].isFile()) {
                    File file = new File(files[idx].getAbsolutePath());
                    byteCounter += file.copyTo(destFile);
                }

                // copy folder
                if (files[idx].isDirectory()) {
                    Folder folder = new Folder(files[idx].getAbsolutePath());
                    byteCounter += folder.copyTo(new Folder(destFile));
                }
            }
        }

        return byteCounter;
    }

    /**
     * <p>Copies this folder object to the given destination folder. Just calls
     * <code>copyTo(Folder)</code> with the <code>Folder</code> created from
     * <code>destination</code>.</p>
     *
     * @param destination where to copy the folder to
     * @return the number of bytes that were copied
     * @throws IOException on problems with the streams
     * @see de.goerntkai.toolbox.fs.File#copyTo(java.lang.String)
     * @see de.goerntkai.toolbox.fs.Folder#copyTo(Folder)
     */
    public final long copyTo(
        final java.io.File destination
    ) throws IOException {

        return this.copyTo(new Folder(destination.getAbsolutePath()));
    }

    /**
     * <p>Copies this folder object to the given destination folder. Just calls
     * <code>copyTo(Folder)</code> with the <code>Folder</code> created from
     * <code>destinationFile</code>.</p>
     *
     * @param destinationPath where to copy the folder to
     * @return the number of bytes that were copied
     * @throws IOException
     * @see de.goerntkai.toolbox.fs.File#copyTo(java.lang.String)
     * @see de.goerntkai.toolbox.fs.Folder#copyTo(Folder)
     *
     */
    /*public long copyTo(String destinationPath) throws IOException {

        return this.copyTo(new Folder(destinationPath));
    }*/

    /**
     * <p>Deletes the folder and recursively all its subdirectories. Returns
     * <code>false</code> if the folder is no directory.</p>
     *
     * @return wether the operation was successful
     */
    public final boolean deleteContents() {

        // initializes the result
        boolean result = true;

        // return false when the folder is no directory
        if (this.isDirectory()) {

            // initializes the array of the contending files and folders
            java.io.File[] files = this.listFiles();

            // iterate over every contending file and folder
            for (int idx = 0; idx < files.length; idx++) {

                // delete contending file
                if (files[idx].isFile()) {
                    result = files[idx].delete();
                }

                // delete subdirectory with contents
                if (files[idx].isDirectory()) {
                    try {
                        Folder folder = new Folder(files[idx]
                            .getAbsolutePath());
                        result = folder.deleteWithContents();

                    } catch (NoFolderException exception) {
                        // ToDo ExceptionTracer.print(exception);
                        exception.printStackTrace();
                    }
                }
            }

        } else {
            // the folder is no directory
            result = false;
        }

        return result;
    }

    /**
     * <p>Deletes recursively all the subdirectories, the contending files and
     * at last the folder itself.</p>
     *
     * @return wether the operation was successful
     */
    public final boolean deleteWithContents() {

        // delete the contents
        boolean isDeleted = this.deleteContents();

        // delete the folder itself
        if (isDeleted) {
            this.delete();
        }

        return isDeleted;
    }

    /**
     * <p>Returns <code>true</code> when there is at least one subdirectory or
     * <code>false</code> if there is none. If the folder is no directory it
     * will return <code>false</code> as well.</p>
     *
     * @return wether there is a subdirectory
     */
    public final boolean hasSubdirectories() {

        // initalizes the result
        boolean hasSubdirectories = false;

        // initialises the array of the contending files and folders
        java.io.File[] files = this.listFiles();

        // check every contending file or folder if it is a directory
        if (files != null) {
            for (int idx = 0; idx < files.length; idx++) {
                if (files[idx].isDirectory()) {
                    hasSubdirectories = true;
                }
            }
        }

        return hasSubdirectories;
    }

    /**
     * <p>Inherited from <code>java.io.File</code>. Returns an array of
     * <code>java.io.File</code> but the results may be casted to
     * <code>de.goerntkai.toolbox.misc.File</code> or
     * <code>de.goerntkai.toolbox.misc.Foler</code>.</p>
     *
     * @return  an array of castable <code>java.io.File</code>
     * @see java.io.File#listFiles()
     */
    public final java.io.File[] listFiles() {

        // gets the file array from the superclass
        java.io.File[] javaFiles = super.listFiles();

        // creates the result array
        java.io.File[] devbaseFiles = null;

        // do nothing if there are no contents in the Folder
        if (javaFiles != null) {
            devbaseFiles = new java.io.File[javaFiles.length];

            // transforms the results to File or Folder objects
            for (int idx = 0; idx < javaFiles.length; idx++) {
                if (javaFiles[idx].isFile()) {
                    // creates de.goerntkai.toolbox.misc.File
                    devbaseFiles[idx] = new File(javaFiles[idx]
                        .getAbsolutePath());
                }
                if (javaFiles[idx].isDirectory()) {
                    // creates de.goerntkai.toolbox.misc.Flder
                    try {
                        devbaseFiles[idx] = new Folder(javaFiles[idx]
                            .getAbsolutePath());

                    } catch (NoFolderException exception) {
                        // ToDo ExceptionTracer.print(exception);
                        exception.printStackTrace();
                    }
                }
            }
        }

        return devbaseFiles;
    }
}



/* end of file ------------------------------------------------------------ */
