/* ---------------------------------------------------------------------------
created on: 23.06.2004
by: GoerntKai

Copyright: (c) 2004 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */
package de.goerntkai.toolbox.fs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * <p>Reads a complete text file and returns the whole document.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: TextReader.java,v 1.5 2007/05/06 16:25:52 GoerntKai Exp $
 */
public abstract class TextReader {

    /**
     * <p>Holds the message that is thrown with a
     * <code>FileNotFoundException</code>.</p>
     */
    private static final String FILE_DOES_NOT_EXIST_MSG = (
        " does not exist or is no file!" //$NON-NLS-1$
    );

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private TextReader() {

        // nop
    }

    /**
     * <p>Prints out the information about an exception to the standard
     * output.</p>
     *
     * @param exception the exception that was caught
     * @param documentFile the file to which the exception occured
     * @param reader the <code>FileReader</code> to which the exception occured
     * @param inText the text that was already read
     */
    private static void printExceptionInformation(final Exception exception,
        final File documentFile, final FileReader reader,
        final StringBuffer inText) {

        System.out.println();
        // ToDo ExceptionTracer.print(exception);
        exception.printStackTrace();
        System.out.println(
            "documentFile=" + //$NON-NLS-1$
            documentFile.toString()
        );
        System.out.println("reader=" + reader.toString()); //$NON-NLS-1$
        System.out.println("inText=" + inText); //$NON-NLS-1$
        System.out.println();
    }

    /**
     * <p>This method uses <code>java.io.FileReader</code> to get the content
     * of the document file. The file comes in byte by byte.</p>
     *
     * @param documentFile the file to read
     * @return the whole text document
     * @throws FileNotFoundException if the file does not exist or is no file
     */
    public static StringBuffer getTextDocument(final File documentFile)
        throws FileNotFoundException {

        StringBuffer inText = new StringBuffer();

        if (documentFile.exists() && documentFile.isFile()) {
            FileReader reader = new FileReader(documentFile);
            int character;

            try {
                // reads the file byte by byte
                while ((character = reader.read()) != -1) {
                    inText.append((char) character);
                }
                reader.close();

            } catch (IOException exception) {
                // print the exception information
                printExceptionInformation(
                    exception,
                    documentFile,
                    reader,
                    inText);

            } finally {
                try {
                    // close the reader
                    reader.close();

                } catch (IOException exception) {
                    // if the reader can not be closed, it may not be open
                    // ToDo ExceptionTracer.print(exception);
                    exception.printStackTrace();
                }
            }

        } else {
            throw new FileNotFoundException(
                documentFile.getAbsolutePath() +
                FILE_DOES_NOT_EXIST_MSG
            );

        }

        return inText;
    }

    /**
     * <p>This method just calls <code>getTextDocument(java.io.File)</code> to
     * get the content of the document file.</p>
     *
     * @param documentFile the name of the file to read
     * @return the whole text document
     * @throws FileNotFoundException if the file does not exist or is no file
     * @see TextReader#getTextDocument(File)
     */
    public static StringBuffer getTextDocument(
        final String documentFile
    ) throws FileNotFoundException {

        return getTextDocument(new File(documentFile));
    }

    /**
     * <p>Reads the file and stores every line in an array.</p>
     * <dl>
     *  <dt>Precondition:</dt>
     *  <dd>theFile != null</dd>
     * </dl>
     *
     * @param theFile the file to read
     * @return an array of lines
     * @throws FileNotFoundException if the file does not exist or is no file
     */
    public static String[] getTextLines(final java.io.File theFile)
        throws FileNotFoundException {

        assert theFile != null : "The file is null!"; //$NON-NLS-1$

        if (!theFile.exists() || !theFile.isFile()) {
            String message = (
                "[" + //$NON-NLS-1$
                theFile.getAbsolutePath() +
                "] doesn't exist or is not a file!" //$NON-NLS-1$
            );
            throw new FileNotFoundException(message);
        }

        BufferedReader reader = null;
        String line = null;
        ArrayList lines = new ArrayList();

        try {
            if (theFile.isFile()) {
                reader = new BufferedReader(new FileReader(theFile));
                while ((line = reader.readLine()) != null) {
                    lines.add(line);
                }
            }

        } catch (IOException exception) {
            // ToDo ExceptionTracer.print(exception);
            exception.printStackTrace();

        } finally {
            try {
                if (reader != null) {
                    reader.close();
                }
            } catch (IOException exception) {
                // ToDo ExceptionTracer.print(exception);
                exception.printStackTrace();
            }
        }

        String[] result = new String[lines.size()];
        for (int idx = 0; idx < result.length; idx++) {
            result[idx] = (String) lines.get(idx);
        }

        return result;
    }

    /**
     * <p>Reads the file and stores every line in an array.</p>
     * <dl>
     *  <dt>Precondition:</dt>
     *  <dd>theFile != null</dd>
     * </dl>
     *
     * @param theFile the file to read
     * @return an array of lines
     * @throws FileNotFoundException if the file does not exist or is no file
     */
    public static String[] getTextLines(final String theFile)
        throws FileNotFoundException {

        return getTextLines(new File(theFile));
    }
}



/* end of file ------------------------------------------------------------ */
