/* ---------------------------------------------------------------------------
created on: 29.09.2004
    by: GoerntKai

Copyright: (c) 2004 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */
package de.goerntkai.toolbox.fs;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * <p>Writes the complete text in one file.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: TextWriter.java,v 1.5 2007/05/06 16:25:52 GoerntKai Exp $
 */
public abstract class TextWriter {

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private TextWriter() {

        // nop
    }

    /**
     * <p>Prints out the information about an exception to the standard
     * output.</p>
     *
     * @param exception the exception that was caught
     * @param documentFile the file to which the exception occured
     * @param writer the <code>FileWriter</code> to which the exception occured
     * @param text the text that should have been written
     */
    private static void printExceptionInformation(
        final Exception exception,
        final File documentFile,
        final FileWriter writer,
        final String text
    ) {

        System.out.println();
        // ToDo ExceptionTracer.print(exception);
        exception.printStackTrace();
        System.out.println(
            "documentFile=" + //$NON-NLS-1$
            documentFile.toString()
        );
        System.out.println("writer=" + writer.toString()); //$NON-NLS-1$
        System.out.println("text=" + text); //$NON-NLS-1$
        System.out.println();
    }

    /**
     * <p>This method uses <code>FileWriter</code> to write the content of the
     * document file.</p>
     *
     * @param documentFile the file to write to
     * @param text the content of the document
     */
    public static void setTextDocument(
        final File documentFile,
        final String text
    ) {

        FileWriter writer = null;

        try {
            // writes the file
            writer = new FileWriter(documentFile);
            writer.write(text.toString());
            writer.close();

        } catch (IOException exception) {
            // print the exception information
            printExceptionInformation(exception, documentFile, writer, text);

        } finally {
            try {
                // close the writer
                if (writer != null) {
                    writer.close();
                }

            } catch (IOException exception) {
                // if the writer can not be closed, it may not be open
                // ToDo ExceptionTracer.print(exception);
                exception.printStackTrace();
            }
        }
    }

    /**
     * <p>This method just calls
     * <code>setTextDocument(java.io.File, java.lang.String)</code> to write
     * the content of the document file.</p>
     *
     * @param documentFile the file to write to
     * @param text the content of the document
     * @see TextWriter#setTextDocument(File, String)
     */
    public static void setTextDocument(
        final File documentFile,
        final StringBuffer text
    ) {

        setTextDocument(documentFile, text.toString());
    }

    /**
     * <p>This method just calls
     * <code>setTextDocument(java.io.File, java.lang.String)</code> to write
     * the content of the document file.</p>
     *
     * @param documentFile the name of the file to write to
     * @param text the content of the document
     * @see TextWriter#setTextDocument(File, String)
     */
    public static void setTextDocument(
        final String documentFile,
        final String text
    ) {

        setTextDocument(new File(documentFile), text);
    }

    /**
     * <p>This method just calls
     * <code>setTextDocument(java.io.File, java.lang.String)</code> to write
     * the content of the document file.</p>
     *
     * @param documentFile the name of the file to write to
     * @param text the content of the document
     * @see TextWriter#setTextDocument(File, String)
     */
    public static void setTextDocument(
        final String documentFile,
        final StringBuffer text
    ) {

        setTextDocument(new File(documentFile), text.toString());
    }

    /**
     * <p>Writes every element of the array to a new line.</p>
     * <dl>
     *  <dt>Precondition:</dt>
     *  <dd>theFile != null</dd>
     * </dl>
     *
     * @param theFile the file to write to (must not be <code>null</code>)
     * @param textLines the lines to write
     */
    public static void setTextLines(
        final java.io.File theFile,
        final String[] textLines
    ) {

        assert theFile != null : "The file is null!"; //$NON-NLS-1$

        BufferedWriter writer = null;

        try {
            writer = new BufferedWriter(new FileWriter(theFile));
            for (int idx = 0; idx < textLines.length; idx++) {
                String line = textLines[idx];
                writer.write(line);
                if (
                    line.length() > 0 &&
                    line.charAt(line.length() - 1) != '\n'
                ) {
                    writer.newLine();
                }
            }

        } catch (IOException exception) {
            // ToDo ExceptionTracer.print(exception);
            exception.printStackTrace();

        } finally {
            try {
                if (writer != null) {
                    writer.close();
                }
            } catch (IOException exception) {
                // ToDo ExceptionTracer.print(exception);
                exception.printStackTrace();
            }
        }
    }

    /**
     * <p>Writes every element of the array to a new line.</p>
     * <dl>
     *  <dt>Precondition:</dt>
     *  <dd>theFile != null</dd>
     * </dl>
     *
     * @param theFile the file to write to (must not be <code>null</code>)
     * @param textLines the lines to write
     */
    public static void setTextLines(
        final String theFile,
        final String[] textLines
    ) {

        setTextLines(new File(theFile), textLines);
    }
}



/* end of file ------------------------------------------------------------ */
