/* ---------------------------------------------------------------------------
created on: 29.04.2005
    by: GoerntKai

Copyright: (c) 2005 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */
package de.goerntkai.toolbox.log;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.logging.Formatter;
import java.util.logging.Level;
import java.util.logging.LogRecord;

import de.goerntkai.toolbox.misc.StringUtil;

/**
 * <p>The <code>FileFormatter</code> displays almost any information coming
 * from the log record. The information which is changing the least often is
 * found on the leftmost position. The information changing most often is found
 * on the rightmost position. That helps keeping the columns in line and gives
 * a better overview when reading the log.</p>
 *
 * <p>The parameterlist is only displayed in case that the log level is
 * <code>ALL</code>, <code>FINER</code>, <code>FINEST</code>,
 * <code>WARNING</code> or <code>SEVERE</code>.</p>
 *
 * <p>Any given <code>Throwable</code> is displayed only by its message. In
 * case that the log level is <code>ALL</code>, <code>FINEST</code>,
 * <code>WARNING</code> or <code>SEVERE</code> the <code>Throwable</code> will
 * be listed with its stack trace trailing.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: FileFormatter.java,v 1.4 2007/04/29 17:54:03 GoerntKai Exp $
 */
public class FileFormatter extends Formatter {

    /**
     * <p>Standard Constructor.</p>
     */
    public FileFormatter() {

        super();
    }

    /**
     * <p>Formats the logger name.</p>
     *
     * @param theRecord to format
     * @return the formatted logger name
     */
    private String formatLoggerName(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds logger name
        output.append('[');
        theRecord.getLoggerName();
        output.append(']');

        return output.toString();
    }

    /**
     * <p>Formats the log level.</p>
     *
     * @param theRecord to format
     * @return the formatted log level
     */
    private String formatLogLevel(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds log level
        output.append('[');
        output.append(theRecord.getLevel());
        output.append(']');

        return output.toString();
    }

    /**
     * <p>Formats the log message.</p>
     *
     * @param theRecord to format
     * @return the formatted log message
     */
    private String formatLogMessage(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds log message
        output.append(" - "); //$NON-NLS-1$
        output.append(theRecord.getMessage());

        return output.toString();
    }

    /**
     * <p>Formats the procedure call.</p>
     *
     * @param theRecord to format
     * @return the formatted procedure call
     */
    private String formatProcedureCall(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds procedure call
        output.append('[');
        output.append(theRecord.getSourceClassName());
        output.append('.');
        output.append(theRecord.getSourceMethodName());
        output.append('(');

        Level level = theRecord.getLevel();
        if (
            Level.ALL.equals(level) ||
            Level.FINER.equals(level) ||
            Level.FINEST.equals(level) ||
            Level.WARNING.equals(level) ||
            Level.SEVERE.equals(level)
        ) {

            Object[] parameters = theRecord.getParameters();
            for (int idx = 0; idx < parameters.length; idx++) {
                if (idx > 0) {
                    output.append(", "); //$NON-NLS-1$
                }
                output.append(parameters[idx]);
            }
        }

        output.append(")]"); //$NON-NLS-1$

        return output.toString();
    }

    /**
     * <p>Formats the sequence number.</p>
     *
     * @param theRecord to format
     * @return the formatted sequence number
     */
    private String formatSequenceNumber(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds sequence number
        output.append('[');
        output.append(theRecord.getSequenceNumber());
        output.append(']');

        return output.toString();
    }

    /**
     * <p>Formats the stacktrace recursively.</p>
     *
     * @param theIndentation of the stacktrace
     * @param theThrowable to format the stacktrace of
     * @return the formatted stacktrace
     */
    private String formatStacktrace(final String theIndentation,
        final Throwable theThrowable) {

        StringBuffer output = new StringBuffer();

        // adds stacktrace
        output.append("\n"); //$NON-NLS-1$
        output.append(theIndentation);
        output.append(theThrowable);
        output.append("\n"); //$NON-NLS-1$

        String myIndent = "  " + theIndentation; //$NON-NLS-1$
        StackTraceElement[] stackTrace = theThrowable.getStackTrace();
        for (int idx = 0; idx < stackTrace.length; idx++) {
            output.append("  "); //$NON-NLS-1$
            output.append(myIndent);
            output.append(stackTrace[idx].toString());
            output.append("\n"); //$NON-NLS-1$
        }

        Throwable cause = theThrowable.getCause();
        if (cause != null) {
            output.append(this.formatStacktrace(myIndent, cause));
        }

        return output.toString();
    }

    /**
     * <p>Formats the thread ID.</p>
     *
     * @param theRecord to format
     * @return the formatted thread ID
     */
    private String formatThreadID(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds thread ID
        output.append('[');
        theRecord.getThreadID();
        output.append(']');

        return output.toString();
    }

    /**
     * <p>Formats the <code>Throwable</code>.</p>
     *
     * @param theRecord to format
     * @return the formatted <code>Throwable</code>
     */
    private String formatThrowable(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds Throwable
        Throwable thrown = theRecord.getThrown();
        if (thrown != null) {
            Level level = theRecord.getLevel();
            if (
                Level.ALL.equals(level) ||
                Level.FINEST.equals(level) ||
                Level.WARNING.equals(level) ||
                Level.SEVERE.equals(level)
            ) {

                output.append(this.formatStacktrace("", thrown)); //$NON-NLS-1$
            } else {
                output.append(" - "); //$NON-NLS-1$
                output.append(thrown.getMessage());
            }
        }

        return output.toString();
    }

    /**
     * <p>Formats the timestamp.</p>
     *
     * @param theRecord to format
     * @return the formatted timestamp
     */
    private String formatTimestamp(final LogRecord theRecord) {

        final int lead2 = 2;
        final int lead3 = 3;

        StringBuffer output = new StringBuffer();

        // adds timestamp
        Date timestamp = new Date(theRecord.getMillis());
        GregorianCalendar gTimestamp = new GregorianCalendar();
        gTimestamp.setTime(timestamp);
        output.append('[');
        output.append(gTimestamp.get(Calendar.YEAR));
        output.append('-');
        output.append(StringUtil.leadWithZeroes((gTimestamp
            .get(Calendar.MONTH) + 1), lead2));
        output.append('-');
        output.append(StringUtil.leadWithZeroes(gTimestamp
            .get(Calendar.DAY_OF_MONTH), lead2));
        output.append(" - "); //$NON-NLS-1$
        output.append(StringUtil.leadWithZeroes(gTimestamp
            .get(Calendar.HOUR_OF_DAY), lead2));
        output.append(':');
        output.append(StringUtil.leadWithZeroes(gTimestamp
            .get(Calendar.MINUTE), lead2));
        output.append(':');
        output.append(StringUtil.leadWithZeroes(gTimestamp
            .get(Calendar.SECOND), lead2));
        output.append('.');
        output.append(StringUtil.leadWithZeroes(gTimestamp
            .get(Calendar.MILLISECOND), lead3));
        output.append(']');

        return output.toString();
    }

    /**
     * <p>Formats the log output.</p>
     *
     * @param theRecord to format
     * @return the log output as a single line
     * @see java.util.logging.Formatter#format(java.util.logging.LogRecord)
     */
    public final synchronized String format(final LogRecord theRecord) {

        StringBuffer output = new StringBuffer();

        // adds sequence number
        output.append(this.formatSequenceNumber(theRecord));

        // adds timestamp
        output.append(this.formatTimestamp(theRecord));

        // adds thread ID
        output.append(this.formatThreadID(theRecord));

        // adds logger name
        output.append(this.formatLoggerName(theRecord));

        // adds log level
        output.append(this.formatLogLevel(theRecord));

        // adds procedure call
        output.append(this.formatProcedureCall(theRecord));

        // adds log message
        output.append(this.formatLogMessage(theRecord));

        // adds Throwable
        output.append(this.formatThrowable(theRecord));

        // not added:
        //record.getResourceBundle();
        //record.getThrown();

        // adds new line
        output.append("\n"); //$NON-NLS-1$

        return output.toString();
    }
}



/* end of file ------------------------------------------------------------ */
