/* ---------------------------------------------------------------------------
created on: 04.02.2006
    by: GoerntKai

Copyright: (c) 2006 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
Based on http://www.javapractices.com/Topic17.cjp.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.misc;

import java.util.Arrays;

/**
 * <p>Collected methods which allow easy implementation of <code>equals</code>.
 * Example use case in a class called Car:</p>
 *
 * <pre>
 * public boolean equals(Object that) {
 *
 *     if (this == that) {
 *         return true;
 *     }
 *     if (!(that instanceof Car)) {
 *         return false;
 *     }
 *     Car thatCar = (Car)that;
 *     return EqualsUtil.areEqual(this.fName, that.fName)
 *         &amp;&amp; EqualsUtil.areEqual(this.fNumDoors, that.fNumDoors)
 *         &amp;&amp; EqualsUtil.areEqual(this.fGasMileage, that.fGasMileage)
 *         &amp;&amp; EqualsUtil.areEqual(this.fColor, that.fColor)
 *         &amp;&amp; Arrays.equals(this.fMaintenanceChecks,
 *             that.fMaintenanceChecks); //array!
 * }
 * </pre>
 *
 * <p>Equality of arrays is delegated to the <code>Arrays.equals</code>
 * methods.</p>
 *
 * @author <a href="mailto:kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: EqualsUtil.java,v 1.3 2007/04/29 17:54:03 GoerntKai Exp $
 */
public abstract class EqualsUtil {

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private EqualsUtil() {

        // nop
    }

    /**
     * <p>Compares booleans.</p>
     *
     * @param aThis boolean
     * @param aThat boolean
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final boolean aThis, final boolean aThat) {

        return aThis == aThat;
    }

    /**
     * <p>Compares boolean arrays.</p>
     *
     * @param aThis boolean array
     * @param aThat boolean array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(
        final boolean[] aThis, final boolean[] aThat
    ) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares byte arrays.</p>
     *
     * @param aThis byte array
     * @param aThat byte array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final byte[] aThis, final byte[] aThat) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares characters.</p>
     *
     * @param aThis character
     * @param aThat character
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final char aThis, final char aThat) {

        return aThis == aThat;
    }

    /**
     * <p>Compares character arrays.</p>
     *
     * @param aThis character array
     * @param aThat character array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final char[] aThis, final char[] aThat) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares doubles.</p>
     *
     * @param aThis double
     * @param aThat double
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final double aThis, final double aThat) {

        return Double.doubleToLongBits(aThis) == Double.doubleToLongBits(
            aThat
        );
    }

    /**
     * <p>Compares double arrays.</p>
     *
     * @param aThis double array
     * @param aThat double array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(
        final double[] aThis,
        final double[] aThat
    ) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares floats.</p>
     *
     * @param aThis float
     * @param aThat float
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final float aThis, final float aThat) {

        return Float.floatToIntBits(aThis) == Float.floatToIntBits(aThat);
    }

    /**
     * <p>Compares float arrays.</p>
     *
     * @param aThis float array
     * @param aThat float array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final float[] aThis, final float[] aThat) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares integer arrays.</p>
     *
     * @param aThis integer array
     * @param aThat integer array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final int[] aThis, final int[] aThat) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares longs.</p>
     *
     * <p>Note that byte, short, and int are handled by this method, through
     * implicit conversion.</p>
     *
     * @param aThis long
     * @param aThat long
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final long aThis, final long aThat) {

        return aThis == aThat;
    }

    /**
     * <p>Compares long arrays.</p>
     *
     * @param aThis long array
     * @param aThat long array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final long[] aThis, final long[] aThat) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares objects.</p>
     *
     * <p>Possibly-null object field. Includes type-safe enumerations and
     * collections, but does not include arrays. See class comment.</p>
     *
     * @param aThis object
     * @param aThat object
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final Object aThis, final Object aThat) {

        boolean result = false;

        if (aThis == null) {
            result = (aThat == null);
        } else {
            result = aThis.equals(aThat);
        }

        return result;
    }

    /**
     * <p>Compares object arrays.</p>
     *
     * @param aThis object array
     * @param aThat object array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(
        final Object[] aThis,
        final Object[] aThat
    ) {

        return Arrays.equals(aThis, aThat);
    }

    /**
     * <p>Compares short arrays.</p>
     *
     * @param aThis short array
     * @param aThat short array
     * @return <code>true</code> or <code>false</code>
     */
    public static boolean areEqual(final short[] aThis, final short[] aThat) {

        return Arrays.equals(aThis, aThat);
    }
}



/* end of file ------------------------------------------------------------ */
