/* ---------------------------------------------------------------------------
created on: 04.02.2006
    by: GoerntKai

Copyright: (c) 2005 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
Based on http://www.javapractices.com/Topic28.cjp.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.misc;

import java.lang.reflect.Array;

/**
 * Collected methods which allow easy implementation of <code>hashCode</code>.
 * Example use case:
 *
 * <pre>
 * public int hashCode() {
 *
 *     int result = HashCodeUtil.SEED;
 *     // collect the contributions of various fields
 *     result = HashCodeUtil.hash(result, fPrimitive);
 *     result = HashCodeUtil.hash(result, fObject);
 *     result = HashCodeUtil.hash(result, fArray);
 *     return result;
 * }
 * </pre>
 *
 * @author <a href="mailto:kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: HashCodeUtil.java,v 1.3 2007/04/29 17:54:03 GoerntKai Exp $
 */
public abstract class HashCodeUtil {

    /**
     * <p>Holds the length of a <code>long</code>.</p>
     */
    private static final int LONG_LENGTH = 32;

    /**
     * Prime.
     */
    private static final int ODD_PRIME_NUMBER = 37;

    /**
     * An initial value for a <code>hashCode</code>, to which is added
     * contributions from fields. Using a non-zero value decreases collisons of
     * <code>hashCode</code> values.
     */
    public static final int SEED = 23;

    /**
     * <p>Suppresses external instantiation.</p>
     */
    private HashCodeUtil() {

        // nop
    }

    /**
     * <p>Returns the first Term.</p>
     *
     * @param aSeed to begin with
     * @return the first term
     */
    private static int firstTerm(final int aSeed) {

        return ODD_PRIME_NUMBER * aSeed;
    }

    /**
     * <p>Returns wether the object is an array or not.</p>
     *
     * @param aObject to create the code for
     * @return <code>true</code> or <code>false</code>
     */
    private static boolean isArray(final Object aObject) {

        return aObject.getClass().isArray();
    }

    /**
     * Booleans.
     *
     * @param aSeed for the code
     * @param aBoolean to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final boolean aBoolean) {

        int result = 0;

        if (aBoolean) {
            result = firstTerm(aSeed) + 1;
        } else {
            result = firstTerm(aSeed);
        }

        return result;
    }

    /**
     * Characters.
     *
     * @param aSeed for the code
     * @param aChar to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final char aChar) {

        return firstTerm(aSeed) + aChar;
    }

    /**
     * Doubles.
     *
     * @param aSeed for the code
     * @param aDouble to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final double aDouble) {

        return hash(aSeed, Double.doubleToLongBits(aDouble));
    }

    /**
     * Floats.
     *
     * @param aSeed for the code
     * @param aFloat to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final float aFloat) {

        return hash(aSeed, Float.floatToIntBits(aFloat));
    }

    /**
     * Integers.
     *
     * @param aSeed for the code
     * @param aInt to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final int aInt) {

        /*
         * Implementation Note Note that byte and short are handled by this
         * method, through implicit conversion.
         */
        return firstTerm(aSeed) + aInt;
    }

    /**
     * Longs.
     *
     * @param aSeed for the code
     * @param aLong to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final long aLong) {

        return firstTerm(aSeed) + (int) (aLong ^ (aLong >>> LONG_LENGTH));
    }

    /**
     * <code>aObject</code> is a possibly-null object field, and possibly an
     * array. If <code>aObject</code> is an array, then each element may be a
     * primitive or a possibly-null object.
     *
     * @param aSeed for the code
     * @param aObject to create the code for
     * @return hashcode for a boolean
     */
    public static int hash(final int aSeed, final Object aObject) {

        int result = aSeed;

        if (aObject == null) {
            result = hash(result, 0);

        } else if (!isArray(aObject)) {
            result = hash(result, aObject.hashCode());

        } else {
            int length = Array.getLength(aObject);
            for (int idx = 0; idx < length; ++idx) {
                Object item = Array.get(aObject, idx);
                // recursive call!
                result = hash(result, item);
            }
        }

        return result;
    }
}



/* end of file ------------------------------------------------------------ */
