/* ---------------------------------------------------------------------------
created on: 19.08.2006
    by: GoerntKai

Copyright: (c) 2006 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.mock;

import java.util.ArrayList;
import java.util.List;

import junit.framework.Assert;

/**
 * <p>Overrides the <code>Logger</code> from the Java 1.4 logging framework
 * for testing purposes.</p>
 *
 * <p>Compares the exact messages but checks for wildcards '*' and '?'. The
 * questionmark represents a single character, the asterisk a number of
 * characters.</p>
 *
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: LoggerMock.java,v 1.3 2007/05/06 09:04:38 GoerntKai Exp $
 */
public class LoggerMock extends java.util.logging.Logger {

    /**
     * <p>Holds the index of the current message to check.</p>
     */
    private int index = 0;

    /**
     * <p>Holds the messages.</p>
     */
    private List messages = new ArrayList();

    /**
     * <p>Standard constructor.</p>
     */
    public LoggerMock() {

        super(null, null);
    }

    /**
     * <p>Compares each character of the message ignoring the wildcards.
     * Returns <code>true</code> when all the characters are equal or
     * represented by wildcards, <code>false</code> otherwise.</p>
     *
     * <dl>
     *  <dt>Preconditions</dt>
     *  <dd>theMessage != <code>null</code></dd>
     * </dl>
     *
     * @param theMessage to verify
     * @return wether all characters are equal
     */
    private boolean compareEachCharacter(final String theMessage) {

        boolean result = true;

        String expectedMsg = (String) this.messages.get(this.index++);

        int expectedIdx = 0;
        for (int msgIdx = 0; msgIdx < theMessage.length(); msgIdx++) {
            if (expectedIdx < expectedMsg.length()) {
                char expected = expectedMsg.charAt(expectedIdx);
                switch(expected) {
                    case '?':
                        expectedIdx++;
                        break;

                    case '*':
                        char next = '*';
                        if (expectedIdx + 1 < expectedMsg.length()) {
                            next = expectedMsg.charAt(expectedIdx + 1);
                        }
                        if (next == theMessage.charAt(msgIdx)) {
                            expectedIdx++;
                            msgIdx--;
                        }
                        break;

                    default:
                        if (expected != theMessage.charAt(msgIdx)) {
                            result = false;
                        }
                        expectedIdx++;
                        break;
                }
            } else {
                Assert.fail(
                    "the message no. [" + //$NON-NLS-1$
                    (this.index - 1) +
                    "] expected <" + //$NON-NLS-1$
                    expectedMsg +
                    "> but was <" + //$NON-NLS-1$
                    theMessage +
                    ">" //$NON-NLS-1$
                );
            }
        }

        return result;
    }

    /**
     * <p>Delegatee method of all logging methods (info, fine and so on).</p>
     *
     * @param theMessage to verify
     */
    private void log(final String theMessage) {

        // too many messages is only a failure not an error
        Assert.assertTrue(
            "too many messages [" + //$NON-NLS-1$
                theMessage +
                "]", //$NON-NLS-1$
            this.index < this.messages.size()
        );

        // checks for wildcards
        Assert.assertTrue(
            "the message no. [" + //$NON-NLS-1$
                this.index +
                "] expected <" + //$NON-NLS-1$
                this.messages.get(this.index) +
                "> but was <" + //$NON-NLS-1$
                theMessage +
                ">", //$NON-NLS-1$
            this.compareEachCharacter(theMessage)
        );
    }

    /**
     * <p>Adds an expected message to the list.</p>
     *
     * @param theMessage to add
     */
    public final void addMessage(final String theMessage) {

        this.messages.add(theMessage);
    }

    /**
     * <p>Checks config messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#config(java.lang.String)
     */
    public final void config(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Checks fine messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#fine(java.lang.String)
     */
    public final void fine(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Checks finer messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#finer(java.lang.String)
     */
    public final void finer(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Checks finest messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#finest(java.lang.String)
     */
    public final void finest(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Returns <code>true</code> when all the expected messages have been
     * sent to the logger. Throws an <code>AssertionFailedError</code>
     * otherwise.</p>
     *
     * @return wether all messages were logged
     */
    public final boolean hasAllMessagesDone() {

        Assert.assertEquals("all messages done", //$NON-NLS-1$
            this.messages.size(),
            this.index);

        return (this.messages.size() == this.index);
    }

    /**
     * <p>Checks info messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#info(java.lang.String)
     */
    public final void info(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Checks severe messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#severe(java.lang.String)
     */
    public final void severe(final String theMessage) {

        this.log(theMessage);
    }

    /**
     * <p>Checks warning messages.</p>
     *
     * @param theMessage to verify
     * @see java.util.logging.Logger#warning(java.lang.String)
     */
    public final void warning(final String theMessage) {

        this.log(theMessage);
    }
}



/* end of file ------------------------------------------------------------ */
