/* ---------------------------------------------------------------------------
created on: 04.02.2006
    by: GoerntKai

Copyright: (c) 2006 by Kai Grnt (kai.goernt@dimensionswandler.de).
All rights reserved.
--------------------------------------------------------------------------- */

package de.goerntkai.toolbox.time;

import java.util.Date;

import de.goerntkai.toolbox.misc.EqualsUtil;
import de.goerntkai.toolbox.misc.HashCodeUtil;

/**
 * <p>Measures elapsed time.</p>
 *
 * <p>The moment <code>StopWatch</code> is instantiated it beginns counting.
 * Every time <code>getElapsedMillis</code> is called it returns the time
 * elapsed since instantiation in milliseconds. The clock does not stop
 * counting so any subsequent call of <code>getElapsedMillis</code> returns
 * the time elapsed since instantiation. If the start of a new counting is
 * needed <code>reset</code> has to be called.</p>
 *
 * <p>The <code>StopWatch</code> is thread safe.</p>
 *
 * <dl>
 * <dt>Invariants</dt>
 * <dd>this.startTime != <code>null</code></dd>
 * </dl>
 *
 * @see de.goerntkai.toolbox.time.Clock
 * @author <a href="kai.goernt@dimensionswandler.de">Kai Grnt</a>
 * @version $Id: StopWatch.java,v 1.2 2007/04/29 17:54:04 GoerntKai Exp $
 */
class StopWatch implements Clock {

    /**
     * <p>Holds the time when <code>StopWatch</code> starts.</p>
     */
    private Date startTime = null;

    /**
     * <p>Standard constructor sets the starting time.</p>
     */
    public StopWatch() {

        super();
        this.startTime = new Date();

        // invariants
        this.checkInvariants();
    }

    /**
     * <p>Checks the invariants. As there are so many it is encapsuled in this
     * method.</p>
     */
    private void checkInvariants() {

        // invariants
        assert this.startTime != null :
            "this.startTime is [null]"; //$NON-NLS-1$
    }

    /**
     * <p><code>StopWatch</code>es are equal when <code>this.startTime</code>
     * equals <code>other.startTime</code>.</p>
     *
     * <p>This method is mainly contained for testing.</p>
     *
     * <p>Implemented on recommendation of <a
     * href="http://www.javapractices.com/Topic17.cjp"
     * >http://www.javapractices.com/Topic17.cjp</a>.</p>
     *
     * @param otherObject the object to compare with
     * @return <code>true</code> if this object is the same as
     *      <code>other</code> and <code>false</code> otherwise
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(final Object otherObject) {

        // invariants
        this.checkInvariants();

        // superclass object checks identity
        if (super.equals(otherObject)) {
            return true;
        }

        // returns false when other object is null; see rule no. 5
        // checks symmetry
        if (!(otherObject instanceof StopWatch)) {
            return false;
        }

        // casts other to this class
        StopWatch other = (StopWatch) otherObject;

        // invariants
        this.checkInvariants();

        return EqualsUtil.areEqual(this.startTime, other.startTime);
    }

    /**
     * <p>Returns the elapsed time in milliseconds.</p>
     *
     * <dl>
     * <dt>Postconditions</dt>
     * <dd>this.starTime.getTime() >= this.getElapsedMillis()</dd>
     * </dl>
     *
     * @return elapsed milliseconds
     * @see de.goerntkai.toolbox.time.Clock#getElapsedMillis()
     */
    public synchronized long getElapsedMillis() {

        // invariants
        this.checkInvariants();

        Date endTime = new Date();
        long millis = endTime.getTime() - this.startTime.getTime();

        // postconditions
        assert this.startTime.getTime() >= millis :
            "this.startTime is < this.getElapsedMillis()"; //$NON-NLS-1$

        // invariants
        this.checkInvariants();

        return millis;
    }

    /**
     * <p>Implemented as <code>equals()</code> is implemented.</p>
     *
     * <p>Implemented on recommendation of <a
     * href="http://www.javapractices.com/Topic28.cjp"
     * >http://www.javapractices.com/Topic28.cjp</a>.</p>
     *
     * @return the hashCode
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {

        // invariants
        this.checkInvariants();

        int result = HashCodeUtil.SEED;

        //collect the contributions of various fields
        result = HashCodeUtil.hash(result, this.startTime);

        // invariants
        this.checkInvariants();

        return result;
    }

    /**
     * <p>Resets the starting time.</p>
     *
     * <dl>
     * <dt>Postconditions</dt>
     * <dd>this.getElapsedMillis() == 0</dd>
     * </dl>
     *
     * @see de.goerntkai.toolbox.time.Clock#reset()
     */
    public synchronized void reset() {

        // invariants
        this.checkInvariants();

        this.startTime = null;
        this.startTime = new Date();

        // postconditions
        assert this.getElapsedMillis() == 0 :
            "this.getElapsedMillis is > 0"; //$NON-NLS-1$

        // invariants
        this.checkInvariants();
    }

    /**
     * <p>This method is mainly contained for debugging. It returns
     * <code>super.toString()</code> concatenated with the fields and its
     * values.</p>
     *
     * @return the String representation of the object
     * @see java.lang.Object#toString()
     */
    public String toString() {

        // invariants
        this.checkInvariants();

        return (
            super.toString() +
            "[" + //$NON-NLS-1$
            "startTime=" + //$NON-NLS-1$
            this.startTime.toString() +
            "]" //$NON-NLS-1$
        );
    }
}



/* end of file ------------------------------------------------------------ */
